#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'coverdialogbase.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COVERDIALOGBASE_H
#define UI_COVERDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include "covericonview.h"
#include "k3listview.h"
#include "klineedit.h"
#include "klistwidget.h"

QT_BEGIN_NAMESPACE

class Ui_CoverDialogBase
{
public:
    QHBoxLayout *hboxLayout;
    KListWidget *m_artists;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout1;
    KLineEdit *m_searchLine;
    CoverIconView *m_covers;

    void setupUi(QWidget *CoverDialogBase)
    {
        if (CoverDialogBase->objectName().isEmpty())
            CoverDialogBase->setObjectName(QString::fromUtf8("CoverDialogBase"));
        CoverDialogBase->resize(685, 554);
        hboxLayout = new QHBoxLayout(CoverDialogBase);
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        m_artists = new KListWidget(CoverDialogBase);
        m_artists->setObjectName(QString::fromUtf8("m_artists"));
        QSizePolicy sizePolicy(static_cast<QSizePolicy::Policy>(5), static_cast<QSizePolicy::Policy>(7));
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_artists->sizePolicy().hasHeightForWidth());
        m_artists->setSizePolicy(sizePolicy);
        m_artists->setMinimumSize(QSize(164, 0));

        hboxLayout->addWidget(m_artists);

        vboxLayout = new QVBoxLayout();
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        hboxLayout1 = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        m_searchLine = new KLineEdit(CoverDialogBase);
        m_searchLine->setObjectName(QString::fromUtf8("m_searchLine"));
        QSizePolicy sizePolicy1(static_cast<QSizePolicy::Policy>(5), static_cast<QSizePolicy::Policy>(0));
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_searchLine->sizePolicy().hasHeightForWidth());
        m_searchLine->setSizePolicy(sizePolicy1);

        hboxLayout1->addWidget(m_searchLine);


        vboxLayout->addLayout(hboxLayout1);

        m_covers = new CoverIconView(CoverDialogBase);
        m_covers->setObjectName(QString::fromUtf8("m_covers"));
        QSizePolicy sizePolicy2(static_cast<QSizePolicy::Policy>(7), static_cast<QSizePolicy::Policy>(7));
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(m_covers->sizePolicy().hasHeightForWidth());
        m_covers->setSizePolicy(sizePolicy2);

        vboxLayout->addWidget(m_covers);


        hboxLayout->addLayout(vboxLayout);


        retranslateUi(CoverDialogBase);

        QMetaObject::connectSlotsByName(CoverDialogBase);
    } // setupUi

    void retranslateUi(QWidget *CoverDialogBase)
    {
        CoverDialogBase->setWindowTitle(tr2i18n("Cover Manager", 0));
    } // retranslateUi

};

namespace Ui {
    class CoverDialogBase: public Ui_CoverDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // COVERDIALOGBASE_H

