#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'filerenameroptionsbase.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILERENAMEROPTIONSBASE_H
#define UI_FILERENAMEROPTIONSBASE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QGridLayout>
#include <QtGui/QGroupBox>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QRadioButton>
#include <QtGui/QSpacerItem>
#include <QtGui/QSpinBox>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include <klineedit.h>

QT_BEGIN_NAMESPACE

class Ui_FileRenamerTagOptionsBase
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *m_tagFormatGroup;
    QVBoxLayout *vboxLayout1;
    QLabel *m_description;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    KLineEdit *m_prefixText;
    QLabel *m_tagLabel;
    KLineEdit *m_suffixText;
    QSpacerItem *spacerItem1;
    QLabel *m_substitution;
    QGroupBox *m_emptyTagGroup;
    QVBoxLayout *vboxLayout2;
    QRadioButton *m_includeEmptyButton;
    QRadioButton *m_ignoreTagButton;
    QHBoxLayout *hboxLayout1;
    QRadioButton *m_useValueButton;
    KLineEdit *m_emptyTagValue;
    QGroupBox *m_trackGroup;
    QVBoxLayout *vboxLayout3;
    QLabel *textLabel10;
    QHBoxLayout *hboxLayout2;
    QHBoxLayout *hboxLayout3;
    QLabel *textLabel5;
    QSpinBox *m_trackWidth;
    QGridLayout *gridLayout;
    QLabel *m_doubleDigitExample;
    QLabel *m_singleDigitExample;
    QLabel *m_singleDigit;
    QLabel *m_doubleDigit;
    QSpacerItem *spacerItem2;

    void setupUi(QWidget *FileRenamerTagOptionsBase)
    {
        if (FileRenamerTagOptionsBase->objectName().isEmpty())
            FileRenamerTagOptionsBase->setObjectName(QString::fromUtf8("FileRenamerTagOptionsBase"));
        FileRenamerTagOptionsBase->resize(915, 377);
        vboxLayout = new QVBoxLayout(FileRenamerTagOptionsBase);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        vboxLayout->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        m_tagFormatGroup = new QGroupBox(FileRenamerTagOptionsBase);
        m_tagFormatGroup->setObjectName(QString::fromUtf8("m_tagFormatGroup"));
        vboxLayout1 = new QVBoxLayout(m_tagFormatGroup);
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        vboxLayout1->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        m_description = new QLabel(m_tagFormatGroup);
        m_description->setObjectName(QString::fromUtf8("m_description"));
        m_description->setTextFormat(Qt::RichText);
        m_description->setAlignment(Qt::AlignCenter);
        m_description->setWordWrap(true);

        vboxLayout1->addWidget(m_description);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        m_prefixText = new KLineEdit(m_tagFormatGroup);
        m_prefixText->setObjectName(QString::fromUtf8("m_prefixText"));
        m_prefixText->setAlignment(Qt::AlignRight);

        hboxLayout->addWidget(m_prefixText);

        m_tagLabel = new QLabel(m_tagFormatGroup);
        m_tagLabel->setObjectName(QString::fromUtf8("m_tagLabel"));

        hboxLayout->addWidget(m_tagLabel);

        m_suffixText = new KLineEdit(m_tagFormatGroup);
        m_suffixText->setObjectName(QString::fromUtf8("m_suffixText"));
        m_suffixText->setAlignment(Qt::AlignLeading);

        hboxLayout->addWidget(m_suffixText);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem1);


        vboxLayout1->addLayout(hboxLayout);

        m_substitution = new QLabel(m_tagFormatGroup);
        m_substitution->setObjectName(QString::fromUtf8("m_substitution"));
        QFont font;
        font.setFamily(QString::fromUtf8("Sans Serif"));
        font.setPointSize(9);
        font.setBold(false);
        font.setItalic(true);
        font.setUnderline(false);
        font.setWeight(50);
        font.setStrikeOut(false);
        m_substitution->setFont(font);
        m_substitution->setAlignment(Qt::AlignCenter);

        vboxLayout1->addWidget(m_substitution);


        vboxLayout->addWidget(m_tagFormatGroup);

        m_emptyTagGroup = new QGroupBox(FileRenamerTagOptionsBase);
        m_emptyTagGroup->setObjectName(QString::fromUtf8("m_emptyTagGroup"));
        vboxLayout2 = new QVBoxLayout(m_emptyTagGroup);
#ifndef UI_Q_OS_MAC
        vboxLayout2->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        vboxLayout2->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        m_includeEmptyButton = new QRadioButton(m_emptyTagGroup);
        m_includeEmptyButton->setObjectName(QString::fromUtf8("m_includeEmptyButton"));

        vboxLayout2->addWidget(m_includeEmptyButton);

        m_ignoreTagButton = new QRadioButton(m_emptyTagGroup);
        m_ignoreTagButton->setObjectName(QString::fromUtf8("m_ignoreTagButton"));
        m_ignoreTagButton->setChecked(true);

        vboxLayout2->addWidget(m_ignoreTagButton);

        hboxLayout1 = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        m_useValueButton = new QRadioButton(m_emptyTagGroup);
        m_useValueButton->setObjectName(QString::fromUtf8("m_useValueButton"));

        hboxLayout1->addWidget(m_useValueButton);

        m_emptyTagValue = new KLineEdit(m_emptyTagGroup);
        m_emptyTagValue->setObjectName(QString::fromUtf8("m_emptyTagValue"));
        m_emptyTagValue->setEnabled(false);

        hboxLayout1->addWidget(m_emptyTagValue);


        vboxLayout2->addLayout(hboxLayout1);


        vboxLayout->addWidget(m_emptyTagGroup);

        m_trackGroup = new QGroupBox(FileRenamerTagOptionsBase);
        m_trackGroup->setObjectName(QString::fromUtf8("m_trackGroup"));
        vboxLayout3 = new QVBoxLayout(m_trackGroup);
#ifndef UI_Q_OS_MAC
        vboxLayout3->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        vboxLayout3->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout3->setObjectName(QString::fromUtf8("vboxLayout3"));
        textLabel10 = new QLabel(m_trackGroup);
        textLabel10->setObjectName(QString::fromUtf8("textLabel10"));
        textLabel10->setTextFormat(Qt::RichText);
        textLabel10->setAlignment(Qt::AlignCenter);
        textLabel10->setWordWrap(true);

        vboxLayout3->addWidget(textLabel10);

        hboxLayout2 = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout2->setSpacing(6);
#endif
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        hboxLayout3 = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout3->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout3->setObjectName(QString::fromUtf8("hboxLayout3"));
        textLabel5 = new QLabel(m_trackGroup);
        textLabel5->setObjectName(QString::fromUtf8("textLabel5"));

        hboxLayout3->addWidget(textLabel5);


        hboxLayout2->addLayout(hboxLayout3);

        m_trackWidth = new QSpinBox(m_trackGroup);
        m_trackWidth->setObjectName(QString::fromUtf8("m_trackWidth"));
        m_trackWidth->setMinimum(1);

        hboxLayout2->addWidget(m_trackWidth);

        gridLayout = new QGridLayout();
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        m_doubleDigitExample = new QLabel(m_trackGroup);
        m_doubleDigitExample->setObjectName(QString::fromUtf8("m_doubleDigitExample"));
        m_doubleDigitExample->setFont(font);

        gridLayout->addWidget(m_doubleDigitExample, 1, 1, 1, 1);

        m_singleDigitExample = new QLabel(m_trackGroup);
        m_singleDigitExample->setObjectName(QString::fromUtf8("m_singleDigitExample"));
        m_singleDigitExample->setFont(font);

        gridLayout->addWidget(m_singleDigitExample, 0, 1, 1, 1);

        m_singleDigit = new QLabel(m_trackGroup);
        m_singleDigit->setObjectName(QString::fromUtf8("m_singleDigit"));
        m_singleDigit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(m_singleDigit, 0, 0, 1, 1);

        m_doubleDigit = new QLabel(m_trackGroup);
        m_doubleDigit->setObjectName(QString::fromUtf8("m_doubleDigit"));
        m_doubleDigit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(m_doubleDigit, 1, 0, 1, 1);


        hboxLayout2->addLayout(gridLayout);


        vboxLayout3->addLayout(hboxLayout2);


        vboxLayout->addWidget(m_trackGroup);

        spacerItem2 = new QSpacerItem(546, 16, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout->addItem(spacerItem2);

#ifndef UI_QT_NO_SHORTCUT
        textLabel5->setBuddy(m_trackWidth);
#endif // QT_NO_SHORTCUT

        retranslateUi(FileRenamerTagOptionsBase);
        QObject::connect(m_trackWidth, SIGNAL(valueChanged(int)), FileRenamerTagOptionsBase, SLOT(slotTrackWidthChanged()));
        QObject::connect(m_prefixText, SIGNAL(textChanged(QString)), FileRenamerTagOptionsBase, SLOT(slotBracketsChanged()));
        QObject::connect(m_suffixText, SIGNAL(textChanged(QString)), FileRenamerTagOptionsBase, SLOT(slotBracketsChanged()));
        QObject::connect(m_emptyTagValue, SIGNAL(textChanged(QString)), FileRenamerTagOptionsBase, SLOT(slotEmptyActionChanged()));
        QObject::connect(m_useValueButton, SIGNAL(toggled(bool)), FileRenamerTagOptionsBase, SLOT(slotEmptyActionChanged()));
        QObject::connect(m_ignoreTagButton, SIGNAL(toggled(bool)), FileRenamerTagOptionsBase, SLOT(slotEmptyActionChanged()));
        QObject::connect(m_includeEmptyButton, SIGNAL(toggled(bool)), FileRenamerTagOptionsBase, SLOT(slotEmptyActionChanged()));
        QObject::connect(m_useValueButton, SIGNAL(toggled(bool)), m_emptyTagValue, SLOT(setEnabled(bool)));

        QMetaObject::connectSlotsByName(FileRenamerTagOptionsBase);
    } // setupUi

    void retranslateUi(QWidget *FileRenamerTagOptionsBase)
    {
        m_substitution->setText(tr2i18n("Substitution Example", 0));
        m_includeEmptyButton->setText(tr2i18n("Include in the &filename anyways", 0));
        m_ignoreTagButton->setText(tr2i18n("&Ignore this tag when renaming the file", 0));
        m_useValueButton->setText(tr2i18n("Use &this value:", 0));
        m_emptyTagValue->setText(tr2i18n("Empty", "value to use in replacement when source tag empty"));
        m_trackGroup->setTitle(tr2i18n("Track numbering", 0));
        textLabel10->setText(tr2i18n("JuK can force the track used in a file name to have a minimum number of digits. You may want to do this for better sorting in file managers.", 0));
        textLabel5->setText(tr2i18n("Minimum number of digits:", 0));
        m_doubleDigitExample->setText(tr2i18n("014", 0));
        m_singleDigitExample->setText(tr2i18n("003", 0));
        m_singleDigit->setText(tr2i18n("3 ->", 0));
        m_doubleDigit->setText(tr2i18n("14 ->", 0));
        Q_UNUSED(FileRenamerTagOptionsBase);
    } // retranslateUi

};

namespace Ui {
    class FileRenamerTagOptionsBase: public Ui_FileRenamerTagOptionsBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // FILERENAMEROPTIONSBASE_H

