/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a collectionadaptor -i dbuscollectionproxy.h -l DBusCollectionProxy /usr/src/ports/juk/juk-17.04.3-1.x86_64/src/juk-17.04.3/org.kde.juk.collection.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "collectionadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class CollectionAdaptor
 */

CollectionAdaptor::CollectionAdaptor(DBusCollectionProxy *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

CollectionAdaptor::~CollectionAdaptor()
{
    // destructor
}

void CollectionAdaptor::createPlaylist(const QString &name)
{
    // handle method call org.kde.juk.collection.createPlaylist
    parent()->createPlaylist(name);
}

void CollectionAdaptor::openFile(const QString &playlist, const QStringList &files)
{
    // handle method call org.kde.juk.collection.openFile
    parent()->openFile(playlist, files);
}

void CollectionAdaptor::openFile(const QString &playlist, const QString &file)
{
    // handle method call org.kde.juk.collection.openFile
    parent()->openFile(playlist, file);
}

void CollectionAdaptor::openFile(const QStringList &files)
{
    // handle method call org.kde.juk.collection.openFile
    parent()->openFile(files);
}

void CollectionAdaptor::openFile(const QString &file)
{
    // handle method call org.kde.juk.collection.openFile
    parent()->openFile(file);
}

QString CollectionAdaptor::playingPlaylist()
{
    // handle method call org.kde.juk.collection.playingPlaylist
    return parent()->playingPlaylist();
}

QStringList CollectionAdaptor::playlistTracks(const QString &playlist)
{
    // handle method call org.kde.juk.collection.playlistTracks
    return parent()->playlistTracks(playlist);
}

QStringList CollectionAdaptor::playlists()
{
    // handle method call org.kde.juk.collection.playlists
    return parent()->playlists();
}

void CollectionAdaptor::remove()
{
    // handle method call org.kde.juk.collection.remove
    parent()->remove();
}

void CollectionAdaptor::removeTrack(const QString &playlist, const QStringList &files)
{
    // handle method call org.kde.juk.collection.removeTrack
    parent()->removeTrack(playlist, files);
}

void CollectionAdaptor::setPlaylist(const QString &name)
{
    // handle method call org.kde.juk.collection.setPlaylist
    parent()->setPlaylist(name);
}

QString CollectionAdaptor::trackCover(const QString &track)
{
    // handle method call org.kde.juk.collection.trackCover
    return parent()->trackCover(track);
}

QString CollectionAdaptor::trackProperty(const QString &file, const QString &property)
{
    // handle method call org.kde.juk.collection.trackProperty
    return parent()->trackProperty(file, property);
}

QString CollectionAdaptor::visiblePlaylist()
{
    // handle method call org.kde.juk.collection.visiblePlaylist
    return parent()->visiblePlaylist();
}


#include "collectionadaptor.moc"
