#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'directorylistbase.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIRECTORYLISTBASE_H
#define UI_DIRECTORYLISTBASE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QCheckBox>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QListView>
#include <QtGui/QSpacerItem>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include "kpushbutton.h"

QT_BEGIN_NAMESPACE

class Ui_DirectoryListBase
{
public:
    QVBoxLayout *verticalLayout_5;
    QVBoxLayout *verticalLayout_3;
    QLabel *informationLabel;
    QHBoxLayout *horizontalLayout;
    QListView *directoryListView;
    QVBoxLayout *verticalLayout;
    KPushButton *addDirectoryButton;
    KPushButton *removeDirectoryButton;
    QSpacerItem *rightColumnSpacer;
    QVBoxLayout *verticalLayout_4;
    QLabel *ExcludeInformationLabel;
    QHBoxLayout *horizontalLayout_2;
    QListView *excludeDirectoryListView;
    QVBoxLayout *verticalLayout_2;
    KPushButton *addExcludeDirectoryButton;
    KPushButton *removeExcludeDirectoryButton;
    QSpacerItem *rightExcludeColumnSpacer;
    QCheckBox *importPlaylistsCheckBox;

    void setupUi(QWidget *DirectoryListBase)
    {
        if (DirectoryListBase->objectName().isEmpty())
            DirectoryListBase->setObjectName(QString::fromUtf8("DirectoryListBase"));
        DirectoryListBase->resize(440, 332);
        verticalLayout_5 = new QVBoxLayout(DirectoryListBase);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        informationLabel = new QLabel(DirectoryListBase);
        informationLabel->setObjectName(QString::fromUtf8("informationLabel"));
        informationLabel->setTextFormat(Qt::PlainText);
        informationLabel->setAlignment(Qt::AlignVCenter);

        verticalLayout_3->addWidget(informationLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        directoryListView = new QListView(DirectoryListBase);
        directoryListView->setObjectName(QString::fromUtf8("directoryListView"));
        directoryListView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        directoryListView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        directoryListView->setAlternatingRowColors(true);
        directoryListView->setSelectionMode(QAbstractItemView::MultiSelection);
        directoryListView->setTextElideMode(Qt::ElideMiddle);
        directoryListView->setLayoutMode(QListView::Batched);
        directoryListView->setUniformItemSizes(true);

        horizontalLayout->addWidget(directoryListView);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        addDirectoryButton = new KPushButton(DirectoryListBase);
        addDirectoryButton->setObjectName(QString::fromUtf8("addDirectoryButton"));

        verticalLayout->addWidget(addDirectoryButton);

        removeDirectoryButton = new KPushButton(DirectoryListBase);
        removeDirectoryButton->setObjectName(QString::fromUtf8("removeDirectoryButton"));

        verticalLayout->addWidget(removeDirectoryButton);

        rightColumnSpacer = new QSpacerItem(158, 48, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(rightColumnSpacer);


        horizontalLayout->addLayout(verticalLayout);


        verticalLayout_3->addLayout(horizontalLayout);


        verticalLayout_5->addLayout(verticalLayout_3);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        ExcludeInformationLabel = new QLabel(DirectoryListBase);
        ExcludeInformationLabel->setObjectName(QString::fromUtf8("ExcludeInformationLabel"));
        ExcludeInformationLabel->setTextFormat(Qt::PlainText);
        ExcludeInformationLabel->setAlignment(Qt::AlignVCenter);

        verticalLayout_4->addWidget(ExcludeInformationLabel);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        excludeDirectoryListView = new QListView(DirectoryListBase);
        excludeDirectoryListView->setObjectName(QString::fromUtf8("excludeDirectoryListView"));
        excludeDirectoryListView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        excludeDirectoryListView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        excludeDirectoryListView->setAlternatingRowColors(true);
        excludeDirectoryListView->setSelectionMode(QAbstractItemView::MultiSelection);
        excludeDirectoryListView->setTextElideMode(Qt::ElideMiddle);
        excludeDirectoryListView->setLayoutMode(QListView::Batched);
        excludeDirectoryListView->setUniformItemSizes(true);

        horizontalLayout_2->addWidget(excludeDirectoryListView);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        addExcludeDirectoryButton = new KPushButton(DirectoryListBase);
        addExcludeDirectoryButton->setObjectName(QString::fromUtf8("addExcludeDirectoryButton"));

        verticalLayout_2->addWidget(addExcludeDirectoryButton);

        removeExcludeDirectoryButton = new KPushButton(DirectoryListBase);
        removeExcludeDirectoryButton->setObjectName(QString::fromUtf8("removeExcludeDirectoryButton"));

        verticalLayout_2->addWidget(removeExcludeDirectoryButton);

        rightExcludeColumnSpacer = new QSpacerItem(158, 48, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(rightExcludeColumnSpacer);


        horizontalLayout_2->addLayout(verticalLayout_2);


        verticalLayout_4->addLayout(horizontalLayout_2);


        verticalLayout_5->addLayout(verticalLayout_4);

        importPlaylistsCheckBox = new QCheckBox(DirectoryListBase);
        importPlaylistsCheckBox->setObjectName(QString::fromUtf8("importPlaylistsCheckBox"));
        importPlaylistsCheckBox->setChecked(true);

        verticalLayout_5->addWidget(importPlaylistsCheckBox);


        retranslateUi(DirectoryListBase);

        QMetaObject::connectSlotsByName(DirectoryListBase);
    } // setupUi

    void retranslateUi(QWidget *DirectoryListBase)
    {
        informationLabel->setText(tr2i18n("Please choose the folders where you keep your music:", 0));
#ifndef UI_QT_NO_TOOLTIP
        directoryListView->setToolTip(tr2i18n("These folders will be scanned on startup for new files.", 0));
#endif // QT_NO_TOOLTIP
        addDirectoryButton->setText(tr2i18n("Add Folder...", 0));
        removeDirectoryButton->setText(tr2i18n("Remove Folder", 0));
        ExcludeInformationLabel->setText(tr2i18n("Please choose the folders that should be excluded from music search:", 0));
#ifndef UI_QT_NO_TOOLTIP
        excludeDirectoryListView->setToolTip(tr2i18n("These folders will be scanned on startup for new files.", 0));
#endif // QT_NO_TOOLTIP
        addExcludeDirectoryButton->setText(tr2i18n("Add Folder...", 0));
        removeExcludeDirectoryButton->setText(tr2i18n("Remove Folder", 0));
        importPlaylistsCheckBox->setText(tr2i18n("Import playlists", 0));
        Q_UNUSED(DirectoryListBase);
    } // retranslateUi

};

namespace Ui {
    class DirectoryListBase: public Ui_DirectoryListBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // DIRECTORYLISTBASE_H

