#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'deletedialogbase.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DELETEDIALOGBASE_H
#define UI_DELETEDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QCheckBox>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QListWidget>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DeleteDialogBase
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *ddWarningIcon;
    QVBoxLayout *vboxLayout1;
    QLabel *textLabel1;
    QLabel *ddDeleteText;
    QListWidget *ddFileList;
    QLabel *ddNumFiles;
    QCheckBox *ddShouldDelete;

    void setupUi(QWidget *DeleteDialogBase)
    {
        if (DeleteDialogBase->objectName().isEmpty())
            DeleteDialogBase->setObjectName(QString::fromUtf8("DeleteDialogBase"));
        DeleteDialogBase->resize(542, 374);
        DeleteDialogBase->setMinimumSize(QSize(420, 320));
        vboxLayout = new QVBoxLayout(DeleteDialogBase);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        ddWarningIcon = new QLabel(DeleteDialogBase);
        ddWarningIcon->setObjectName(QString::fromUtf8("ddWarningIcon"));
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ddWarningIcon->sizePolicy().hasHeightForWidth());
        ddWarningIcon->setSizePolicy(sizePolicy);
        ddWarningIcon->setWordWrap(false);

        hboxLayout->addWidget(ddWarningIcon);

        vboxLayout1 = new QVBoxLayout();
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        textLabel1 = new QLabel(DeleteDialogBase);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        textLabel1->setAlignment(Qt::AlignCenter);
        textLabel1->setWordWrap(false);

        vboxLayout1->addWidget(textLabel1);

        ddDeleteText = new QLabel(DeleteDialogBase);
        ddDeleteText->setObjectName(QString::fromUtf8("ddDeleteText"));
        ddDeleteText->setAlignment(Qt::AlignCenter);
        ddDeleteText->setWordWrap(true);

        vboxLayout1->addWidget(ddDeleteText);


        hboxLayout->addLayout(vboxLayout1);


        vboxLayout->addLayout(hboxLayout);

        ddFileList = new QListWidget(DeleteDialogBase);
        ddFileList->setObjectName(QString::fromUtf8("ddFileList"));
        ddFileList->setSelectionMode(QAbstractItemView::NoSelection);

        vboxLayout->addWidget(ddFileList);

        ddNumFiles = new QLabel(DeleteDialogBase);
        ddNumFiles->setObjectName(QString::fromUtf8("ddNumFiles"));
        ddNumFiles->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ddNumFiles->setWordWrap(false);

        vboxLayout->addWidget(ddNumFiles);

        ddShouldDelete = new QCheckBox(DeleteDialogBase);
        ddShouldDelete->setObjectName(QString::fromUtf8("ddShouldDelete"));

        vboxLayout->addWidget(ddShouldDelete);


        retranslateUi(DeleteDialogBase);
        QObject::connect(ddShouldDelete, SIGNAL(toggled(bool)), DeleteDialogBase, SLOT(slotShouldDelete(bool)));

        QMetaObject::connectSlotsByName(DeleteDialogBase);
    } // setupUi

    void retranslateUi(QWidget *DeleteDialogBase)
    {
        ddWarningIcon->setText(tr2i18n("Icon Placeholder, not in GUI", "KDE::DoNotExtract"));
        textLabel1->setText(tr2i18n("Are you sure that you want to remove these items?", 0));
        ddDeleteText->setText(tr2i18n("Deletion method placeholder, never shown to user.", "KDE::DoNotExtract"));
        ddNumFiles->setText(tr2i18n("Placeholder for number of files, not in GUI", "KDE::DoNotExtract"));
#ifndef UI_QT_NO_TOOLTIP
        ddShouldDelete->setToolTip(tr2i18n("If checked, files will be permanently removed instead of being placed in the Trash Bin", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        ddShouldDelete->setWhatsThis(tr2i18n("<qt><p>If this box is checked, files will be <b>permanently removed</b> instead of being placed in the Trash Bin.</p>\n"
"\n"
"<p><em>Use this option with caution</em>: Most filesystems are unable to reliably undelete deleted files.</p></qt>", 0));
#endif // QT_NO_WHATSTHIS
        ddShouldDelete->setText(tr2i18n("&Delete files instead of moving them to the trash", 0));
        Q_UNUSED(DeleteDialogBase);
    } // retranslateUi

};

namespace Ui {
    class DeleteDialogBase: public Ui_DeleteDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // DELETEDIALOGBASE_H

