#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'tageditor.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TAGEDITOR_H
#define UI_TAGEDITOR_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QCheckBox>
#include <QtGui/QFrame>
#include <QtGui/QGridLayout>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QSpacerItem>
#include <QtGui/QWidget>
#include "kcombobox.h"
#include "klineedit.h"
#include "knuminput.h"
#include "ktextedit.h"

QT_BEGIN_NAMESPACE

class Ui_TagEditor
{
public:
    QHBoxLayout *horizontalLayout;
    QGridLayout *tagEditorLayout;
    QLabel *fileNameLabel;
    QLabel *trackLabel;
    QLabel *artistNameLabel;
    QLabel *albumNameLabel;
    QLabel *genreLabel;
    KLineEdit *fileNameBox;
    QLabel *yearLabel;
    KLineEdit *lengthBox;
    KLineEdit *bitrateBox;
    QLabel *lengthLabel;
    QLabel *bitrateLabel;
    QLabel *commentLabel;
    KTextEdit *commentBox;
    QSpacerItem *verticalSpacer;
    KComboBox *albumNameBox;
    KComboBox *genreBox;
    KComboBox *artistNameBox;
    KIntSpinBox *trackSpin;
    KIntSpinBox *yearSpin;
    QFrame *line;
    QLabel *trackNameLabel;
    QCheckBox *artistNameEnable;
    KLineEdit *trackNameBox;
    QCheckBox *trackNameEnable;
    QCheckBox *albumNameEnable;
    QCheckBox *genreEnable;
    QCheckBox *trackEnable;
    QCheckBox *yearEnable;
    QCheckBox *commentEnable;
    QFrame *line_2;

    void setupUi(QWidget *TagEditor)
    {
        if (TagEditor->objectName().isEmpty())
            TagEditor->setObjectName(QString::fromUtf8("TagEditor"));
        TagEditor->resize(622, 221);
        horizontalLayout = new QHBoxLayout(TagEditor);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        tagEditorLayout = new QGridLayout();
        tagEditorLayout->setObjectName(QString::fromUtf8("tagEditorLayout"));
        fileNameLabel = new QLabel(TagEditor);
        fileNameLabel->setObjectName(QString::fromUtf8("fileNameLabel"));

        tagEditorLayout->addWidget(fileNameLabel, 0, 3, 1, 7);

        trackLabel = new QLabel(TagEditor);
        trackLabel->setObjectName(QString::fromUtf8("trackLabel"));

        tagEditorLayout->addWidget(trackLabel, 2, 3, 1, 1);

        artistNameLabel = new QLabel(TagEditor);
        artistNameLabel->setObjectName(QString::fromUtf8("artistNameLabel"));

        tagEditorLayout->addWidget(artistNameLabel, 0, 0, 1, 1);

        albumNameLabel = new QLabel(TagEditor);
        albumNameLabel->setObjectName(QString::fromUtf8("albumNameLabel"));

        tagEditorLayout->addWidget(albumNameLabel, 4, 0, 1, 1);

        genreLabel = new QLabel(TagEditor);
        genreLabel->setObjectName(QString::fromUtf8("genreLabel"));

        tagEditorLayout->addWidget(genreLabel, 6, 0, 1, 1);

        fileNameBox = new KLineEdit(TagEditor);
        fileNameBox->setObjectName(QString::fromUtf8("fileNameBox"));

        tagEditorLayout->addWidget(fileNameBox, 1, 3, 1, 7);

        yearLabel = new QLabel(TagEditor);
        yearLabel->setObjectName(QString::fromUtf8("yearLabel"));

        tagEditorLayout->addWidget(yearLabel, 2, 7, 1, 1);

        lengthBox = new KLineEdit(TagEditor);
        lengthBox->setObjectName(QString::fromUtf8("lengthBox"));
        lengthBox->setReadOnly(true);

        tagEditorLayout->addWidget(lengthBox, 3, 4, 1, 1);

        bitrateBox = new KLineEdit(TagEditor);
        bitrateBox->setObjectName(QString::fromUtf8("bitrateBox"));
        bitrateBox->setReadOnly(true);

        tagEditorLayout->addWidget(bitrateBox, 3, 8, 1, 1);

        lengthLabel = new QLabel(TagEditor);
        lengthLabel->setObjectName(QString::fromUtf8("lengthLabel"));

        tagEditorLayout->addWidget(lengthLabel, 3, 3, 1, 1);

        bitrateLabel = new QLabel(TagEditor);
        bitrateLabel->setObjectName(QString::fromUtf8("bitrateLabel"));

        tagEditorLayout->addWidget(bitrateLabel, 3, 7, 1, 1);

        commentLabel = new QLabel(TagEditor);
        commentLabel->setObjectName(QString::fromUtf8("commentLabel"));

        tagEditorLayout->addWidget(commentLabel, 4, 3, 1, 2);

        commentBox = new KTextEdit(TagEditor);
        commentBox->setObjectName(QString::fromUtf8("commentBox"));

        tagEditorLayout->addWidget(commentBox, 5, 3, 4, 7);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        tagEditorLayout->addItem(verticalSpacer, 8, 0, 1, 1);

        albumNameBox = new KComboBox(TagEditor);
        albumNameBox->setObjectName(QString::fromUtf8("albumNameBox"));
        albumNameBox->setEditable(true);

        tagEditorLayout->addWidget(albumNameBox, 5, 0, 1, 2);

        genreBox = new KComboBox(TagEditor);
        genreBox->setObjectName(QString::fromUtf8("genreBox"));
        genreBox->setEditable(true);

        tagEditorLayout->addWidget(genreBox, 7, 0, 1, 2);

        artistNameBox = new KComboBox(TagEditor);
        artistNameBox->setObjectName(QString::fromUtf8("artistNameBox"));
        artistNameBox->setEditable(true);

        tagEditorLayout->addWidget(artistNameBox, 1, 0, 1, 2);

        trackSpin = new KIntSpinBox(TagEditor);
        trackSpin->setObjectName(QString::fromUtf8("trackSpin"));
        trackSpin->setMaximum(999);

        tagEditorLayout->addWidget(trackSpin, 2, 4, 1, 1);

        yearSpin = new KIntSpinBox(TagEditor);
        yearSpin->setObjectName(QString::fromUtf8("yearSpin"));
        yearSpin->setMaximum(9999);

        tagEditorLayout->addWidget(yearSpin, 2, 8, 1, 1);

        line = new QFrame(TagEditor);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::VLine);
        line->setFrameShadow(QFrame::Sunken);

        tagEditorLayout->addWidget(line, 0, 2, 8, 1);

        trackNameLabel = new QLabel(TagEditor);
        trackNameLabel->setObjectName(QString::fromUtf8("trackNameLabel"));

        tagEditorLayout->addWidget(trackNameLabel, 2, 0, 1, 1);

        artistNameEnable = new QCheckBox(TagEditor);
        artistNameEnable->setObjectName(QString::fromUtf8("artistNameEnable"));

        tagEditorLayout->addWidget(artistNameEnable, 0, 1, 1, 1);

        trackNameBox = new KLineEdit(TagEditor);
        trackNameBox->setObjectName(QString::fromUtf8("trackNameBox"));

        tagEditorLayout->addWidget(trackNameBox, 3, 0, 1, 2);

        trackNameEnable = new QCheckBox(TagEditor);
        trackNameEnable->setObjectName(QString::fromUtf8("trackNameEnable"));

        tagEditorLayout->addWidget(trackNameEnable, 2, 1, 1, 1);

        albumNameEnable = new QCheckBox(TagEditor);
        albumNameEnable->setObjectName(QString::fromUtf8("albumNameEnable"));

        tagEditorLayout->addWidget(albumNameEnable, 4, 1, 1, 1);

        genreEnable = new QCheckBox(TagEditor);
        genreEnable->setObjectName(QString::fromUtf8("genreEnable"));

        tagEditorLayout->addWidget(genreEnable, 6, 1, 1, 1);

        trackEnable = new QCheckBox(TagEditor);
        trackEnable->setObjectName(QString::fromUtf8("trackEnable"));

        tagEditorLayout->addWidget(trackEnable, 2, 5, 1, 1);

        yearEnable = new QCheckBox(TagEditor);
        yearEnable->setObjectName(QString::fromUtf8("yearEnable"));

        tagEditorLayout->addWidget(yearEnable, 2, 9, 1, 1);

        commentEnable = new QCheckBox(TagEditor);
        commentEnable->setObjectName(QString::fromUtf8("commentEnable"));

        tagEditorLayout->addWidget(commentEnable, 4, 5, 1, 5);

        line_2 = new QFrame(TagEditor);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::VLine);
        line_2->setFrameShadow(QFrame::Sunken);

        tagEditorLayout->addWidget(line_2, 2, 6, 2, 1);


        horizontalLayout->addLayout(tagEditorLayout);

#ifndef UI_QT_NO_SHORTCUT
        fileNameLabel->setBuddy(fileNameBox);
        trackLabel->setBuddy(trackSpin);
        artistNameLabel->setBuddy(artistNameBox);
        albumNameLabel->setBuddy(albumNameBox);
        genreLabel->setBuddy(genreBox);
        yearLabel->setBuddy(yearSpin);
        lengthLabel->setBuddy(lengthBox);
        bitrateLabel->setBuddy(bitrateBox);
        commentLabel->setBuddy(commentBox);
        trackNameLabel->setBuddy(trackNameBox);
#endif // QT_NO_SHORTCUT

        retranslateUi(TagEditor);
        QObject::connect(artistNameEnable, SIGNAL(toggled(bool)), artistNameBox, SLOT(setEnabled(bool)));
        QObject::connect(trackNameEnable, SIGNAL(toggled(bool)), trackNameBox, SLOT(setEnabled(bool)));
        QObject::connect(albumNameEnable, SIGNAL(toggled(bool)), albumNameBox, SLOT(setEnabled(bool)));
        QObject::connect(genreEnable, SIGNAL(toggled(bool)), genreBox, SLOT(setEnabled(bool)));
        QObject::connect(trackEnable, SIGNAL(toggled(bool)), trackSpin, SLOT(setEnabled(bool)));
        QObject::connect(commentEnable, SIGNAL(toggled(bool)), commentBox, SLOT(setEnabled(bool)));
        QObject::connect(yearEnable, SIGNAL(toggled(bool)), yearSpin, SLOT(setEnabled(bool)));

        QMetaObject::connectSlotsByName(TagEditor);
    } // setupUi

    void retranslateUi(QWidget *TagEditor)
    {
        fileNameLabel->setText(tr2i18n("F&ile name:", 0));
        trackLabel->setText(tr2i18n("T&rack:", 0));
        artistNameLabel->setText(tr2i18n("&Artist name:", 0));
        albumNameLabel->setText(tr2i18n("Album &name:", 0));
        genreLabel->setText(tr2i18n("&Genre:", 0));
        yearLabel->setText(tr2i18n("&Year:", 0));
        lengthLabel->setText(tr2i18n("&Length:", 0));
        bitrateLabel->setText(tr2i18n("&Bitrate:", 0));
        commentLabel->setText(tr2i18n("&Comment:", 0));
        trackNameLabel->setText(tr2i18n("Trac&k name:", 0));
        artistNameEnable->setText(tr2i18n("Enable", 0));
        trackNameEnable->setText(tr2i18n("Enable", 0));
        albumNameEnable->setText(tr2i18n("Enable", 0));
        genreEnable->setText(tr2i18n("Enable", 0));
        trackEnable->setText(tr2i18n("Enable", 0));
        yearEnable->setText(tr2i18n("Enable", 0));
        commentEnable->setText(tr2i18n("Enable", 0));
        Q_UNUSED(TagEditor);
    } // retranslateUi

};

namespace Ui {
    class TagEditor: public Ui_TagEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TAGEDITOR_H

