/****************************************************************************
** Meta object code from reading C++ file 'mediaplayer2.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/juk-17.04.3/mpris2/mediaplayer2.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mediaplayer2.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_MediaPlayer2[] = {

 // content:
       6,       // revision
       0,       // classname
       1,   14, // classinfo
       2,   16, // methods
       9,   26, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
      36,   13,

 // slots: signature, parameters, type, tag, flags
      63,   62,   62,   52, 0x0a,
      71,   62,   62,   52, 0x0a,

 // properties: name, type, flags
      83,   78, 0x01095001,
      92,   78, 0x01095001,
     100,   78, 0x01095001,
     117,   78, 0x01095001,
     128,   78, 0x01095001,
     149,  141, 0x0a095001,
     158,  141, 0x0a095001,
     183,  171, 0x0b095001,
     203,  171, 0x0b095001,

       0        // eod
};

static const char qt_meta_stringdata_MediaPlayer2[] = {
    "MediaPlayer2\0org.mpris.MediaPlayer2\0"
    "D-Bus Interface\0Q_NOREPLY\0\0Raise()\0"
    "Quit()\0bool\0CanRaise\0CanQuit\0"
    "CanSetFullscreen\0Fullscreen\0HasTrackList\0"
    "QString\0Identity\0DesktopEntry\0QStringList\0"
    "SupportedUriSchemes\0SupportedMimeTypes\0"
};

void MediaPlayer2::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        MediaPlayer2 *_t = static_cast<MediaPlayer2 *>(_o);
        switch (_id) {
        case 0: _t->Raise(); break;
        case 1: _t->Quit(); break;
        default: ;
        }
    }
    Q_UNUSED(_a);
}

const QMetaObjectExtraData MediaPlayer2::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject MediaPlayer2::staticMetaObject = {
    { &QDBusAbstractAdaptor::staticMetaObject, qt_meta_stringdata_MediaPlayer2,
      qt_meta_data_MediaPlayer2, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &MediaPlayer2::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *MediaPlayer2::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *MediaPlayer2::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_MediaPlayer2))
        return static_cast<void*>(const_cast< MediaPlayer2*>(this));
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int MediaPlayer2::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
      else if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = CanRaise(); break;
        case 1: *reinterpret_cast< bool*>(_v) = CanQuit(); break;
        case 2: *reinterpret_cast< bool*>(_v) = CanSetFullscreen(); break;
        case 3: *reinterpret_cast< bool*>(_v) = Fullscreen(); break;
        case 4: *reinterpret_cast< bool*>(_v) = HasTrackList(); break;
        case 5: *reinterpret_cast< QString*>(_v) = Identity(); break;
        case 6: *reinterpret_cast< QString*>(_v) = DesktopEntry(); break;
        case 7: *reinterpret_cast< QStringList*>(_v) = SupportedUriSchemes(); break;
        case 8: *reinterpret_cast< QStringList*>(_v) = SupportedMimeTypes(); break;
        }
        _id -= 9;
    } else if (_c == QMetaObject::WriteProperty) {
        _id -= 9;
    } else if (_c == QMetaObject::ResetProperty) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 9;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_END_MOC_NAMESPACE
