#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'filerenamerbase.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILERENAMERBASE_H
#define UI_FILERENAMERBASE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QFrame>
#include <QtGui/QGridLayout>
#include <QtGui/QGroupBox>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QScrollArea>
#include <QtGui/QSpacerItem>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include <klineedit.h>
#include "kcombobox.h"
#include "kpushbutton.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_FileRenamerBase
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    KComboBox *m_separator;
    QLabel *textLabel1_6;
    QHBoxLayout *hboxLayout;
    KComboBox *m_category;
    KPushButton *m_insertCategory;
    KUrlRequester *m_musicFolder;
    QLabel *textLabel1;
    QLabel *textLabel1_7;
    QFrame *line1_2;
    QScrollArea *m_mainView;
    QWidget *scrollAreaWidgetContents;
    QFrame *line1;
    QGroupBox *groupBox;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem;
    KPushButton *m_showExample;
    QSpacerItem *spacerItem1;
    KLineEdit *m_exampleText;

    void setupUi(QWidget *FileRenamerBase)
    {
        if (FileRenamerBase->objectName().isEmpty())
            FileRenamerBase->setObjectName(QString::fromUtf8("FileRenamerBase"));
        FileRenamerBase->resize(489, 579);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(FileRenamerBase->sizePolicy().hasHeightForWidth());
        FileRenamerBase->setSizePolicy(sizePolicy);
        vboxLayout = new QVBoxLayout(FileRenamerBase);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        gridLayout = new QGridLayout();
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        gridLayout->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        m_separator = new KComboBox(FileRenamerBase);
        m_separator->setObjectName(QString::fromUtf8("m_separator"));
        m_separator->setEditable(true);

        gridLayout->addWidget(m_separator, 1, 1, 1, 1);

        textLabel1_6 = new QLabel(FileRenamerBase);
        textLabel1_6->setObjectName(QString::fromUtf8("textLabel1_6"));

        gridLayout->addWidget(textLabel1_6, 0, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        m_category = new KComboBox(FileRenamerBase);
        m_category->setObjectName(QString::fromUtf8("m_category"));

        hboxLayout->addWidget(m_category);

        m_insertCategory = new KPushButton(FileRenamerBase);
        m_insertCategory->setObjectName(QString::fromUtf8("m_insertCategory"));

        hboxLayout->addWidget(m_insertCategory);


        gridLayout->addLayout(hboxLayout, 2, 1, 1, 1);

        m_musicFolder = new KUrlRequester(FileRenamerBase);
        m_musicFolder->setObjectName(QString::fromUtf8("m_musicFolder"));

        gridLayout->addWidget(m_musicFolder, 0, 1, 1, 1);

        textLabel1 = new QLabel(FileRenamerBase);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));

        gridLayout->addWidget(textLabel1, 2, 0, 1, 1);

        textLabel1_7 = new QLabel(FileRenamerBase);
        textLabel1_7->setObjectName(QString::fromUtf8("textLabel1_7"));

        gridLayout->addWidget(textLabel1_7, 1, 0, 1, 1);


        vboxLayout->addLayout(gridLayout);

        line1_2 = new QFrame(FileRenamerBase);
        line1_2->setObjectName(QString::fromUtf8("line1_2"));
        line1_2->setFrameShape(QFrame::HLine);
        line1_2->setFrameShadow(QFrame::Sunken);
        line1_2->setFrameShape(QFrame::HLine);

        vboxLayout->addWidget(line1_2);

        m_mainView = new QScrollArea(FileRenamerBase);
        m_mainView->setObjectName(QString::fromUtf8("m_mainView"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(1);
        sizePolicy1.setHeightForWidth(m_mainView->sizePolicy().hasHeightForWidth());
        m_mainView->setSizePolicy(sizePolicy1);
        m_mainView->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 483, 340));
        m_mainView->setWidget(scrollAreaWidgetContents);

        vboxLayout->addWidget(m_mainView);

        line1 = new QFrame(FileRenamerBase);
        line1->setObjectName(QString::fromUtf8("line1"));
        line1->setFrameShape(QFrame::HLine);
        line1->setFrameShadow(QFrame::Sunken);
        line1->setFrameShape(QFrame::HLine);

        vboxLayout->addWidget(line1);

        groupBox = new QGroupBox(FileRenamerBase);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        vboxLayout1 = new QVBoxLayout(groupBox);
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        vboxLayout1->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        hboxLayout1 = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem);

        m_showExample = new KPushButton(groupBox);
        m_showExample->setObjectName(QString::fromUtf8("m_showExample"));

        hboxLayout1->addWidget(m_showExample);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem1);


        vboxLayout1->addLayout(hboxLayout1);

        m_exampleText = new KLineEdit(groupBox);
        m_exampleText->setObjectName(QString::fromUtf8("m_exampleText"));
        m_exampleText->setFocusPolicy(Qt::NoFocus);
        m_exampleText->setAcceptDrops(false);
        m_exampleText->setReadOnly(true);

        vboxLayout1->addWidget(m_exampleText);


        vboxLayout->addWidget(groupBox);

#ifndef UI_QT_NO_SHORTCUT
        textLabel1->setBuddy(m_category);
#endif // QT_NO_SHORTCUT
        QWidget::setTabOrder(m_musicFolder, m_separator);
        QWidget::setTabOrder(m_separator, m_category);
        QWidget::setTabOrder(m_category, m_insertCategory);

        retranslateUi(FileRenamerBase);
        QObject::connect(m_musicFolder, SIGNAL(textChanged(QString)), FileRenamerBase, SLOT(exampleTextChanged()));
        QObject::connect(m_separator, SIGNAL(textChanged(QString)), FileRenamerBase, SLOT(exampleTextChanged()));
        QObject::connect(m_showExample, SIGNAL(clicked()), FileRenamerBase, SLOT(toggleExampleDialog()));
        QObject::connect(m_insertCategory, SIGNAL(clicked()), FileRenamerBase, SLOT(insertCategory()));

        QMetaObject::connectSlotsByName(FileRenamerBase);
    } // setupUi

    void retranslateUi(QWidget *FileRenamerBase)
    {
        FileRenamerBase->setWindowTitle(tr2i18n("File Renamer Configuration", 0));
        textLabel1_6->setText(tr2i18n("Music folder:", 0));
        m_category->clear();
        m_category->insertItems(0, QStringList()
         << tr2i18n("Album Tag", 0)
         << tr2i18n("Artist Tag", 0)
         << tr2i18n("Genre Tag", 0)
         << tr2i18n("Title Tag", 0)
         << tr2i18n("Track Tag", 0)
         << tr2i18n("Year Tag", 0)
        );
        m_insertCategory->setText(tr2i18n("Insert Category", 0));
        textLabel1->setText(tr2i18n("Add category:", 0));
        textLabel1_7->setText(tr2i18n("Separator:", 0));
        groupBox->setTitle(tr2i18n("Example", 0));
        m_showExample->setText(tr2i18n("Show Renamer Test Dialog", 0));
    } // retranslateUi

};

namespace Ui {
    class FileRenamerBase: public Ui_FileRenamerBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // FILERENAMERBASE_H

