/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a playeradaptor -i playermanager.h -l PlayerManager /usr/src/ports/juk/juk-17.04.3-1.x86_64/src/juk-17.04.3/org.kde.juk.player.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "playeradaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class PlayerAdaptor
 */

PlayerAdaptor::PlayerAdaptor(PlayerManager *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

PlayerAdaptor::~PlayerAdaptor()
{
    // destructor
}

void PlayerAdaptor::back()
{
    // handle method call org.kde.juk.player.back
    parent()->back();
}

int PlayerAdaptor::currentTime()
{
    // handle method call org.kde.juk.player.currentTime
    return parent()->currentTime();
}

int PlayerAdaptor::currentTimeMSecs()
{
    // handle method call org.kde.juk.player.currentTimeMSecs
    return parent()->currentTimeMSecs();
}

void PlayerAdaptor::forward()
{
    // handle method call org.kde.juk.player.forward
    parent()->forward();
}

void PlayerAdaptor::mute()
{
    // handle method call org.kde.juk.player.mute
    parent()->mute();
}

void PlayerAdaptor::pause()
{
    // handle method call org.kde.juk.player.pause
    parent()->pause();
}

bool PlayerAdaptor::paused()
{
    // handle method call org.kde.juk.player.paused
    return parent()->paused();
}

void PlayerAdaptor::play(const QString &file)
{
    // handle method call org.kde.juk.player.play
    parent()->play(file);
}

void PlayerAdaptor::play()
{
    // handle method call org.kde.juk.player.play
    parent()->play();
}

void PlayerAdaptor::playPause()
{
    // handle method call org.kde.juk.player.playPause
    parent()->playPause();
}

bool PlayerAdaptor::playing()
{
    // handle method call org.kde.juk.player.playing
    return parent()->playing();
}

QString PlayerAdaptor::playingString()
{
    // handle method call org.kde.juk.player.playingString
    return parent()->playingString();
}

QString PlayerAdaptor::randomPlayMode()
{
    // handle method call org.kde.juk.player.randomPlayMode
    return parent()->randomPlayMode();
}

void PlayerAdaptor::seekBack()
{
    // handle method call org.kde.juk.player.seekBack
    parent()->seekBack();
}

void PlayerAdaptor::seekForward()
{
    // handle method call org.kde.juk.player.seekForward
    parent()->seekForward();
}

void PlayerAdaptor::setRandomPlayMode(const QString &randomMode)
{
    // handle method call org.kde.juk.player.setRandomPlayMode
    parent()->setRandomPlayMode(randomMode);
}

void PlayerAdaptor::setVolume(double volume)
{
    // handle method call org.kde.juk.player.setVolume
    parent()->setVolume(volume);
}

int PlayerAdaptor::status()
{
    // handle method call org.kde.juk.player.status
    return parent()->status();
}

void PlayerAdaptor::stop()
{
    // handle method call org.kde.juk.player.stop
    parent()->stop();
}

int PlayerAdaptor::totalTime()
{
    // handle method call org.kde.juk.player.totalTime
    return parent()->totalTime();
}

int PlayerAdaptor::totalTimeMSecs()
{
    // handle method call org.kde.juk.player.totalTimeMSecs
    return parent()->totalTimeMSecs();
}

QStringList PlayerAdaptor::trackProperties()
{
    // handle method call org.kde.juk.player.trackProperties
    return parent()->trackProperties();
}

QString PlayerAdaptor::trackProperty(const QString &property)
{
    // handle method call org.kde.juk.player.trackProperty
    return parent()->trackProperty(property);
}

double PlayerAdaptor::volume()
{
    // handle method call org.kde.juk.player.volume
    return parent()->volume();
}

void PlayerAdaptor::volumeDown()
{
    // handle method call org.kde.juk.player.volumeDown
    parent()->volumeDown();
}

void PlayerAdaptor::volumeUp()
{
    // handle method call org.kde.juk.player.volumeUp
    parent()->volumeUp();
}


#include "playeradaptor.moc"
