/****************************************************************************
** Meta object code from reading C++ file 'dbuscollectionproxy.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/juk-17.04.3/dbuscollectionproxy.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbuscollectionproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_DBusCollectionProxy[] = {

 // content:
       6,       // revision
       0,       // classname
       1,   14, // classinfo
      14,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
      43,   20,

 // slots: signature, parameters, type, tag, flags
      65,   60,   59,   59, 0x0a,
      89,   83,   59,   59, 0x0a,
     125,  111,   59,   59, 0x0a,
     166,  151,   59,   59, 0x0a,
     204,   59,  196,   59, 0x0a,
     222,   59,  196,   59, 0x0a,
     252,   59,  240,   59, 0x0a,
     273,  264,  240,   59, 0x0a,
     311,  297,  196,   59, 0x0a,
     347,  342,   59,   59, 0x0a,
     371,  342,   59,   59, 0x0a,
     392,   59,   59,   59, 0x0a,
     401,  151,   59,   59, 0x0a,
     440,  434,  196,   59, 0x0a,

       0        // eod
};

static const char qt_meta_stringdata_DBusCollectionProxy[] = {
    "DBusCollectionProxy\0org.kde.juk.collection\0"
    "D-Bus Interface\0\0file\0openFile(QString)\0"
    "files\0openFile(QStringList)\0playlist,file\0"
    "openFile(QString,QString)\0playlist,files\0"
    "openFile(QString,QStringList)\0QString\0"
    "visiblePlaylist()\0playingPlaylist()\0"
    "QStringList\0playlists()\0playlist\0"
    "playlistTracks(QString)\0file,property\0"
    "trackProperty(QString,QString)\0name\0"
    "createPlaylist(QString)\0setPlaylist(QString)\0"
    "remove()\0removeTrack(QString,QStringList)\0"
    "track\0trackCover(QString)\0"
};

void DBusCollectionProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        DBusCollectionProxy *_t = static_cast<DBusCollectionProxy *>(_o);
        switch (_id) {
        case 0: _t->openFile((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->openFile((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 2: _t->openFile((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->openFile((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2]))); break;
        case 4: { QString _r = _t->visiblePlaylist();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 5: { QString _r = _t->playingPlaylist();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 6: { QStringList _r = _t->playlists();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = _r; }  break;
        case 7: { QStringList _r = _t->playlistTracks((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = _r; }  break;
        case 8: { QString _r = _t->trackProperty((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 9: _t->createPlaylist((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->setPlaylist((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->remove(); break;
        case 12: _t->removeTrack((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2]))); break;
        case 13: { QString _r = _t->trackCover((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        default: ;
        }
    }
}

const QMetaObjectExtraData DBusCollectionProxy::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject DBusCollectionProxy::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_DBusCollectionProxy,
      qt_meta_data_DBusCollectionProxy, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &DBusCollectionProxy::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *DBusCollectionProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *DBusCollectionProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_DBusCollectionProxy))
        return static_cast<void*>(const_cast< DBusCollectionProxy*>(this));
    return QObject::qt_metacast(_clname);
}

int DBusCollectionProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}
QT_END_MOC_NAMESPACE
