/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a searchadaptor -i searchwidget.h -l SearchWidget /usr/src/ports/juk/juk-17.04.3-1.x86_64/src/juk-17.04.3/org.kde.juk.search.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "searchadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class SearchAdaptor
 */

SearchAdaptor::SearchAdaptor(SearchWidget *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

SearchAdaptor::~SearchAdaptor()
{
    // destructor
}

QString SearchAdaptor::searchText()
{
    // handle method call org.kde.juk.search.searchText
    return parent()->searchText();
}

void SearchAdaptor::setSearchText(const QString &text)
{
    // handle method call org.kde.juk.search.setSearchText
    parent()->setSearchText(text);
}


#include "searchadaptor.moc"
