#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'tagguesserconfigdlgwidget.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TAGGUESSERCONFIGDLGWIDGET_H
#define UI_TAGGUESSERCONFIGDLGWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QGridLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QSpacerItem>
#include <QtGui/QToolButton>
#include <QtGui/QTreeView>
#include <QtGui/QWidget>
#include "kpushbutton.h"

QT_BEGIN_NAMESPACE

class Ui_TagGuesserConfigDlgWidget
{
public:
    QGridLayout *gridLayout;
    QTreeView *lvSchemes;
    QToolButton *bMoveUp;
    QToolButton *bMoveDown;
    KPushButton *bAdd;
    KPushButton *bModify;
    KPushButton *bRemove;
    QSpacerItem *spacer1;
    QLabel *label;

    void setupUi(QWidget *TagGuesserConfigDlgWidget)
    {
        if (TagGuesserConfigDlgWidget->objectName().isEmpty())
            TagGuesserConfigDlgWidget->setObjectName(QString::fromUtf8("TagGuesserConfigDlgWidget"));
        TagGuesserConfigDlgWidget->resize(671, 530);
        gridLayout = new QGridLayout(TagGuesserConfigDlgWidget);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        lvSchemes = new QTreeView(TagGuesserConfigDlgWidget);
        lvSchemes->setObjectName(QString::fromUtf8("lvSchemes"));
        lvSchemes->setRootIsDecorated(false);
        lvSchemes->setExpandsOnDoubleClick(false);

        gridLayout->addWidget(lvSchemes, 0, 0, 6, 1);

        bMoveUp = new QToolButton(TagGuesserConfigDlgWidget);
        bMoveUp->setObjectName(QString::fromUtf8("bMoveUp"));

        gridLayout->addWidget(bMoveUp, 0, 1, 1, 1);

        bMoveDown = new QToolButton(TagGuesserConfigDlgWidget);
        bMoveDown->setObjectName(QString::fromUtf8("bMoveDown"));

        gridLayout->addWidget(bMoveDown, 0, 3, 1, 1);

        bAdd = new KPushButton(TagGuesserConfigDlgWidget);
        bAdd->setObjectName(QString::fromUtf8("bAdd"));

        gridLayout->addWidget(bAdd, 1, 1, 1, 3);

        bModify = new KPushButton(TagGuesserConfigDlgWidget);
        bModify->setObjectName(QString::fromUtf8("bModify"));

        gridLayout->addWidget(bModify, 2, 1, 1, 3);

        bRemove = new KPushButton(TagGuesserConfigDlgWidget);
        bRemove->setObjectName(QString::fromUtf8("bRemove"));

        gridLayout->addWidget(bRemove, 3, 1, 1, 3);

        spacer1 = new QSpacerItem(20, 130, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(spacer1, 4, 2, 1, 1);

        label = new QLabel(TagGuesserConfigDlgWidget);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 5, 2, 1, 1);


        retranslateUi(TagGuesserConfigDlgWidget);

        QMetaObject::connectSlotsByName(TagGuesserConfigDlgWidget);
    } // setupUi

    void retranslateUi(QWidget *TagGuesserConfigDlgWidget)
    {
#ifndef UI_QT_NO_TOOLTIP
        lvSchemes->setToolTip(tr2i18n("Currently used file name schemes", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        lvSchemes->setWhatsThis(tr2i18n("Here you can see the currently configured file name schemes which the \"Guess Tag Information From File Name\" entry in the song's context menu uses to extract tag information from a file name. Each string may contain one of the following placeholders:<ul>\n"
"<li>%t: Title</li>\n"
"<li>%a: Artist</li>\n"
"<li>%A: Album</li>\n"
"<li>%T: Track</li>\n"
"<li>%c: Comment</li>\n"
"</ul>\n"
"For example, the file name scheme \"[%T] %a - %t\" would match \"[01] Deep Purple - Smoke on the water\" but not \"(Deep Purple) Smoke on the water\". For that second name, you would use the scheme \"(%a) %t\".<p/>\n"
"Note that the order in which the schemes appear in the list is relevant, since the tag guesser will go through the list from the top to the bottom, and use the first matching scheme.", 0));
#endif // QT_NO_WHATSTHIS
#ifndef UI_QT_NO_TOOLTIP
        bMoveUp->setToolTip(tr2i18n("Move scheme up", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        bMoveUp->setWhatsThis(tr2i18n("Press this button to move the currently selected scheme one step upwards.", 0));
#endif // QT_NO_WHATSTHIS
        bMoveUp->setText(QString());
#ifndef UI_QT_NO_TOOLTIP
        bMoveDown->setToolTip(tr2i18n("Move scheme down", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        bMoveDown->setWhatsThis(tr2i18n("Press this button to move the currently selected scheme one step downwards.", 0));
#endif // QT_NO_WHATSTHIS
        bMoveDown->setText(QString());
#ifndef UI_QT_NO_TOOLTIP
        bAdd->setToolTip(tr2i18n("Add a new scheme", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        bAdd->setWhatsThis(tr2i18n("Press this button to add a new file name scheme to the end of the list.", 0));
#endif // QT_NO_WHATSTHIS
        bAdd->setText(tr2i18n("&Add", 0));
#ifndef UI_QT_NO_TOOLTIP
        bModify->setToolTip(tr2i18n("Modify scheme", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        bModify->setWhatsThis(tr2i18n("Press this button to modify the currently selected scheme.", 0));
#endif // QT_NO_WHATSTHIS
        bModify->setText(tr2i18n("&Modify", 0));
#ifndef UI_QT_NO_TOOLTIP
        bRemove->setToolTip(tr2i18n("Remove scheme", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        bRemove->setWhatsThis(tr2i18n("Press this button to remove the currently selected scheme from the list.", 0));
#endif // QT_NO_WHATSTHIS
        bRemove->setText(tr2i18n("&Remove", 0));
        label->setText(tr2i18n("You can use the following placeholders:\n"
"%t: Title\n"
"%A: Album\n"
"%a: Artist\n"
"%T: Track\n"
"%c: Comment", 0));
        Q_UNUSED(TagGuesserConfigDlgWidget);
    } // retranslateUi

};

namespace Ui {
    class TagGuesserConfigDlgWidget: public Ui_TagGuesserConfigDlgWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TAGGUESSERCONFIGDLGWIDGET_H

