/****************************************************************************
** Meta object code from reading C++ file 'mediaplayer2player.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/juk-17.04.3/mpris2/mediaplayer2player.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mediaplayer2player.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_MediaPlayer2Player[] = {

 // content:
       6,       // revision
       0,       // classname
       1,   14, // classinfo
      16,   16, // methods
      15,   96, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
      49,   19,

 // signals: signature, parameters, type, tag, flags
      75,   66,   65,   65, 0x05,

 // slots: signature, parameters, type, tag, flags
     103,   65,   65,   93, 0x0a,
     110,   65,   65,   93, 0x0a,
     121,   65,   65,   93, 0x0a,
     129,   65,   65,   93, 0x0a,
     141,   65,   65,   93, 0x0a,
     148,   65,   65,   93, 0x0a,
     162,  155,   65,   93, 0x0a,
     195,  178,   65,   93, 0x0a,
     238,  234,   65,   93, 0x0a,
     255,   65,   65,   65, 0x08,
     278,   65,   65,   65, 0x08,
     293,   65,   65,   65, 0x08,
     321,  312,   65,   65, 0x08,
     350,  343,   65,   65, 0x08,
     378,  371,   65,   65, 0x08,

 // properties: name, type, flags
     398,  390, 0x0a095001,
     413,  390, 0x0a095103,
     431,  424, 0x06095103,
     441,  436, 0x01095103,
     461,  449, 0x08095001,
     470,  424, 0x06095103,
      66,  477, 0x04095001,
     487,  424, 0x06095001,
     499,  424, 0x06095001,
     511,  436, 0x01095001,
     521,  436, 0x01095001,
     535,  436, 0x01095001,
     543,  436, 0x01095001,
     552,  436, 0x01095001,
     560,  436, 0x01095001,

       0        // eod
};

static const char qt_meta_stringdata_MediaPlayer2Player[] = {
    "MediaPlayer2Player\0org.mpris.MediaPlayer2.Player\0"
    "D-Bus Interface\0\0Position\0Seeked(qlonglong)\0"
    "Q_NOREPLY\0Next()\0Previous()\0Pause()\0"
    "PlayPause()\0Stop()\0Play()\0Offset\0"
    "Seek(qlonglong)\0TrackId,Position\0"
    "SetPosition(QDBusObjectPath,qlonglong)\0"
    "Uri\0OpenUri(QString)\0currentSourceChanged()\0"
    "stateUpdated()\0totalTimeChanged()\0"
    "seekable\0seekableChanged(bool)\0newVol\0"
    "volumeChanged(float)\0newPos\0seeked(int)\0"
    "QString\0PlaybackStatus\0LoopStatus\0"
    "double\0Rate\0bool\0Shuffle\0QVariantMap\0"
    "Metadata\0Volume\0qlonglong\0MinimumRate\0"
    "MaximumRate\0CanGoNext\0CanGoPrevious\0"
    "CanPlay\0CanPause\0CanSeek\0CanControl\0"
};

void MediaPlayer2Player::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        MediaPlayer2Player *_t = static_cast<MediaPlayer2Player *>(_o);
        switch (_id) {
        case 0: _t->Seeked((*reinterpret_cast< qlonglong(*)>(_a[1]))); break;
        case 1: _t->Next(); break;
        case 2: _t->Previous(); break;
        case 3: _t->Pause(); break;
        case 4: _t->PlayPause(); break;
        case 5: _t->Stop(); break;
        case 6: _t->Play(); break;
        case 7: _t->Seek((*reinterpret_cast< qlonglong(*)>(_a[1]))); break;
        case 8: _t->SetPosition((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< qlonglong(*)>(_a[2]))); break;
        case 9: _t->OpenUri((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 10: _t->currentSourceChanged(); break;
        case 11: _t->stateUpdated(); break;
        case 12: _t->totalTimeChanged(); break;
        case 13: _t->seekableChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->volumeChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 15: _t->seeked((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData MediaPlayer2Player::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject MediaPlayer2Player::staticMetaObject = {
    { &QDBusAbstractAdaptor::staticMetaObject, qt_meta_stringdata_MediaPlayer2Player,
      qt_meta_data_MediaPlayer2Player, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &MediaPlayer2Player::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *MediaPlayer2Player::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *MediaPlayer2Player::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_MediaPlayer2Player))
        return static_cast<void*>(const_cast< MediaPlayer2Player*>(this));
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int MediaPlayer2Player::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
#ifndef QT_NO_PROPERTIES
      else if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = PlaybackStatus(); break;
        case 1: *reinterpret_cast< QString*>(_v) = LoopStatus(); break;
        case 2: *reinterpret_cast< double*>(_v) = Rate(); break;
        case 3: *reinterpret_cast< bool*>(_v) = Shuffle(); break;
        case 4: *reinterpret_cast< QVariantMap*>(_v) = Metadata(); break;
        case 5: *reinterpret_cast< double*>(_v) = Volume(); break;
        case 6: *reinterpret_cast< qlonglong*>(_v) = Position(); break;
        case 7: *reinterpret_cast< double*>(_v) = MinimumRate(); break;
        case 8: *reinterpret_cast< double*>(_v) = MaximumRate(); break;
        case 9: *reinterpret_cast< bool*>(_v) = CanGoNext(); break;
        case 10: *reinterpret_cast< bool*>(_v) = CanGoPrevious(); break;
        case 11: *reinterpret_cast< bool*>(_v) = CanPlay(); break;
        case 12: *reinterpret_cast< bool*>(_v) = CanPause(); break;
        case 13: *reinterpret_cast< bool*>(_v) = CanSeek(); break;
        case 14: *reinterpret_cast< bool*>(_v) = CanControl(); break;
        }
        _id -= 15;
    } else if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: setLoopStatus(*reinterpret_cast< QString*>(_v)); break;
        case 2: setRate(*reinterpret_cast< double*>(_v)); break;
        case 3: setShuffle(*reinterpret_cast< bool*>(_v)); break;
        case 5: setVolume(*reinterpret_cast< double*>(_v)); break;
        }
        _id -= 15;
    } else if (_c == QMetaObject::ResetProperty) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 15;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void MediaPlayer2Player::Seeked(qlonglong _t1)const
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(const_cast< MediaPlayer2Player *>(this), &staticMetaObject, 0, _a);
}
QT_END_MOC_NAMESPACE
