#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'exampleoptionsbase.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXAMPLEOPTIONSBASE_H
#define UI_EXAMPLEOPTIONSBASE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QGridLayout>
#include <QtGui/QGroupBox>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QRadioButton>
#include <QtGui/QSpacerItem>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include "klineedit.h"
#include "knuminput.h"
#include "kpushbutton.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_ExampleOptionsBase
{
public:
    QGridLayout *gridLayout;
    QGroupBox *groupBox;
    QVBoxLayout *vboxLayout;
    QRadioButton *m_fileTagsButton;
    KUrlRequester *m_exampleFile;
    QRadioButton *m_manualTagsButton;
    QGroupBox *m_manualGroup;
    QGridLayout *gridLayout1;
    QVBoxLayout *vboxLayout1;
    QLabel *textLabel1;
    QLabel *textLabel2;
    QLabel *textLabel3;
    QLabel *textLabel4;
    QLabel *textLabel5;
    QLabel *textLabel6;
    QVBoxLayout *vboxLayout2;
    KLineEdit *m_exampleTitle;
    KLineEdit *m_exampleArtist;
    KLineEdit *m_exampleAlbum;
    KLineEdit *m_exampleGenre;
    KIntSpinBox *m_exampleTrack;
    KIntSpinBox *m_exampleYear;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *ExampleOptionsBase)
    {
        if (ExampleOptionsBase->objectName().isEmpty())
            ExampleOptionsBase->setObjectName(QString::fromUtf8("ExampleOptionsBase"));
        ExampleOptionsBase->resize(404, 473);
        gridLayout = new QGridLayout(ExampleOptionsBase);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setHorizontalSpacing(6);
        gridLayout->setVerticalSpacing(6);
        gridLayout->setContentsMargins(9, 9, 9, 9);
        groupBox = new QGroupBox(ExampleOptionsBase);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        vboxLayout = new QVBoxLayout(groupBox);
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        m_fileTagsButton = new QRadioButton(groupBox);
        m_fileTagsButton->setObjectName(QString::fromUtf8("m_fileTagsButton"));
        m_fileTagsButton->setChecked(false);

        vboxLayout->addWidget(m_fileTagsButton);

        m_exampleFile = new KUrlRequester(groupBox);
        m_exampleFile->setObjectName(QString::fromUtf8("m_exampleFile"));
        m_exampleFile->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_exampleFile->sizePolicy().hasHeightForWidth());
        m_exampleFile->setSizePolicy(sizePolicy);

        vboxLayout->addWidget(m_exampleFile);

        m_manualTagsButton = new QRadioButton(groupBox);
        m_manualTagsButton->setObjectName(QString::fromUtf8("m_manualTagsButton"));
        m_manualTagsButton->setChecked(true);

        vboxLayout->addWidget(m_manualTagsButton);

        m_manualGroup = new QGroupBox(groupBox);
        m_manualGroup->setObjectName(QString::fromUtf8("m_manualGroup"));
        m_manualGroup->setFlat(true);
        gridLayout1 = new QGridLayout(m_manualGroup);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        gridLayout1->setHorizontalSpacing(6);
        gridLayout1->setVerticalSpacing(6);
        gridLayout1->setContentsMargins(9, 9, 9, 9);
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        textLabel1 = new QLabel(m_manualGroup);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        textLabel1->setWordWrap(false);

        vboxLayout1->addWidget(textLabel1);

        textLabel2 = new QLabel(m_manualGroup);
        textLabel2->setObjectName(QString::fromUtf8("textLabel2"));
        textLabel2->setWordWrap(false);

        vboxLayout1->addWidget(textLabel2);

        textLabel3 = new QLabel(m_manualGroup);
        textLabel3->setObjectName(QString::fromUtf8("textLabel3"));
        textLabel3->setWordWrap(false);

        vboxLayout1->addWidget(textLabel3);

        textLabel4 = new QLabel(m_manualGroup);
        textLabel4->setObjectName(QString::fromUtf8("textLabel4"));
        textLabel4->setWordWrap(false);

        vboxLayout1->addWidget(textLabel4);

        textLabel5 = new QLabel(m_manualGroup);
        textLabel5->setObjectName(QString::fromUtf8("textLabel5"));
        textLabel5->setWordWrap(false);

        vboxLayout1->addWidget(textLabel5);

        textLabel6 = new QLabel(m_manualGroup);
        textLabel6->setObjectName(QString::fromUtf8("textLabel6"));
        textLabel6->setWordWrap(false);

        vboxLayout1->addWidget(textLabel6);


        gridLayout1->addLayout(vboxLayout1, 0, 0, 1, 1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(6);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        m_exampleTitle = new KLineEdit(m_manualGroup);
        m_exampleTitle->setObjectName(QString::fromUtf8("m_exampleTitle"));

        vboxLayout2->addWidget(m_exampleTitle);

        m_exampleArtist = new KLineEdit(m_manualGroup);
        m_exampleArtist->setObjectName(QString::fromUtf8("m_exampleArtist"));

        vboxLayout2->addWidget(m_exampleArtist);

        m_exampleAlbum = new KLineEdit(m_manualGroup);
        m_exampleAlbum->setObjectName(QString::fromUtf8("m_exampleAlbum"));

        vboxLayout2->addWidget(m_exampleAlbum);

        m_exampleGenre = new KLineEdit(m_manualGroup);
        m_exampleGenre->setObjectName(QString::fromUtf8("m_exampleGenre"));

        vboxLayout2->addWidget(m_exampleGenre);

        m_exampleTrack = new KIntSpinBox(m_manualGroup);
        m_exampleTrack->setObjectName(QString::fromUtf8("m_exampleTrack"));
        m_exampleTrack->setValue(1);

        vboxLayout2->addWidget(m_exampleTrack);

        m_exampleYear = new KIntSpinBox(m_manualGroup);
        m_exampleYear->setObjectName(QString::fromUtf8("m_exampleYear"));
        m_exampleYear->setMinimum(1000);
        m_exampleYear->setMaximum(2999);
        m_exampleYear->setValue(2007);

        vboxLayout2->addWidget(m_exampleYear);


        gridLayout1->addLayout(vboxLayout2, 0, 1, 1, 1);


        vboxLayout->addWidget(m_manualGroup);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout->addItem(spacerItem);


        gridLayout->addWidget(groupBox, 0, 0, 1, 1);

        QWidget::setTabOrder(m_exampleFile, m_manualTagsButton);
        QWidget::setTabOrder(m_manualTagsButton, m_exampleTitle);
        QWidget::setTabOrder(m_exampleTitle, m_exampleArtist);
        QWidget::setTabOrder(m_exampleArtist, m_exampleAlbum);
        QWidget::setTabOrder(m_exampleAlbum, m_exampleGenre);
        QWidget::setTabOrder(m_exampleGenre, m_exampleTrack);
        QWidget::setTabOrder(m_exampleTrack, m_exampleYear);

        retranslateUi(ExampleOptionsBase);
        QObject::connect(m_fileTagsButton, SIGNAL(toggled(bool)), m_exampleFile, SLOT(setEnabled(bool)));
        QObject::connect(m_fileTagsButton, SIGNAL(toggled(bool)), ExampleOptionsBase, SLOT(exampleSelectionChanged()));
        QObject::connect(m_manualTagsButton, SIGNAL(toggled(bool)), ExampleOptionsBase, SLOT(exampleSelectionChanged()));
        QObject::connect(m_exampleTitle, SIGNAL(textChanged(QString)), ExampleOptionsBase, SLOT(exampleDataChanged()));
        QObject::connect(m_exampleArtist, SIGNAL(textChanged(QString)), ExampleOptionsBase, SLOT(exampleDataChanged()));
        QObject::connect(m_exampleAlbum, SIGNAL(textChanged(QString)), ExampleOptionsBase, SLOT(exampleDataChanged()));
        QObject::connect(m_exampleGenre, SIGNAL(textChanged(QString)), ExampleOptionsBase, SLOT(exampleDataChanged()));
        QObject::connect(m_exampleTrack, SIGNAL(valueChanged(int)), ExampleOptionsBase, SLOT(exampleDataChanged()));
        QObject::connect(m_exampleYear, SIGNAL(valueChanged(int)), ExampleOptionsBase, SLOT(exampleDataChanged()));

        QMetaObject::connectSlotsByName(ExampleOptionsBase);
    } // setupUi

    void retranslateUi(QWidget *ExampleOptionsBase)
    {
        ExampleOptionsBase->setWindowTitle(tr2i18n("Example", 0));
        groupBox->setTitle(tr2i18n("Example Tag Selection", 0));
        m_fileTagsButton->setText(tr2i18n("Get example tags from this file:", 0));
        m_manualTagsButton->setText(tr2i18n("Enter example tags manually:", 0));
        m_manualGroup->setTitle(tr2i18n("Example Tags", 0));
        textLabel1->setText(tr2i18n("Title:", "song title"));
        textLabel2->setText(tr2i18n("Artist:", 0));
        textLabel3->setText(tr2i18n("Album:", 0));
        textLabel4->setText(tr2i18n("Genre:", 0));
        textLabel5->setText(tr2i18n("Track number:", 0));
        textLabel6->setText(tr2i18n("Year:", 0));
        m_exampleTitle->setText(tr2i18n("Title", "example song title"));
        m_exampleArtist->setText(tr2i18n("Artist", 0));
        m_exampleAlbum->setText(tr2i18n("Album", 0));
        m_exampleGenre->setText(tr2i18n("Genre", 0));
    } // retranslateUi

};

namespace Ui {
    class ExampleOptionsBase: public Ui_ExampleOptionsBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // EXAMPLEOPTIONSBASE_H

